@@ -51,10 +51,19 @@ connection_get_bootproto (GstConnectionDialog *dialog)
 }
 
 static void
-connection_bootproto_init (GstConnectionDialog *dialog)
+connection_set_wireless_key_type (GstConnectionDialog *dialog, GstWirelessKey key_type)
 {
-  gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->bootproto_combo), _("Static IP address"));
-  gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->bootproto_combo), _("DHCP"));
+  gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->key_type_combo),
+			    (key_type == GST_WIRELESS_KEY_ASCII) ? 0 : 1);
+}
+
+static GstBootProto
+connection_get_wireless_key_type (GstConnectionDialog *dialog)
+{
+  gint ret;
+
+  ret = gtk_combo_box_get_active (GTK_COMBO_BOX (dialog->key_type_combo));
+  return (ret == 0) ? GST_WIRELESS_KEY_ASCII : GST_WIRELESS_KEY_HEXADECIMAL;
 }
 
 static void
@@ -177,13 +186,16 @@ static void
 wireless_dialog_prepare (GstConnectionDialog *dialog)
 {
   gchar *essid, *key, *dev;
+  GstWirelessKey key_type;
 
   g_object_get (G_OBJECT (dialog->iface),
 		"iface-essid", &essid,
-		"iface-wep-key",   &key,
+		"iface-wep-key", &key,
+		"iface-wep-key-type", &key_type,
 		"iface-dev", &dev,
 		NULL);
 
+  connection_set_wireless_key_type (dialog, key_type);
   gtk_entry_set_text (GTK_ENTRY (GTK_BIN (dialog->essid)->child), (essid) ? essid : "");
   gtk_entry_set_text (GTK_ENTRY (dialog->wep_key), (key) ? key : "");
 
@@ -200,6 +212,7 @@ wireless_dialog_save (GstConnectionDialog *dialog)
   g_object_set (G_OBJECT (dialog->iface),
 		"iface-essid",   get_entry_text (GTK_BIN (dialog->essid)->child),
 		"iface-wep-key", get_entry_text (dialog->wep_key),
+		"iface-wep-key-type", connection_get_wireless_key_type (dialog),
 		NULL);
 }
 
@@ -248,7 +261,7 @@ static void
 isdn_dialog_prepare (GstConnectionDialog *dialog)
 {
   gchar    *login, *password, *phone_number, *dial_prefix;
-  gboolean  default_gw, persist;
+  gboolean  default_gw, peerdns, persist;
 
   g_object_get (G_OBJECT (dialog->iface),
                 "iface-login", &login,
@@ -256,6 +269,7 @@ isdn_dialog_prepare (GstConnectionDialog *dialog)
                 "iface-phone-number", &phone_number,
                 "iface-dial-prefix", &dial_prefix,
                 "iface-default-gw", &default_gw,
+                "iface-peerdns", &peerdns,
                 "iface-persist", &persist,
                 NULL);
 
@@ -272,6 +286,7 @@ isdn_dialog_prepare (GstConnectionDialog *dialog)
                       (dial_prefix) ? dial_prefix : "");
 
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->default_gw), default_gw);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->peerdns), peerdns);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->persist), persist);
 
   g_free (login);
@@ -289,6 +304,7 @@ isdn_dialog_save (GstConnectionDialog *dialog)
                 "iface-phone-number", get_entry_text (dialog->phone_number),
                 "iface-dial-prefix",  get_entry_text (dialog->dial_prefix),
                 "iface-default-gw",   gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->default_gw)),
+                "iface-peerdns",      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->peerdns)),
                 "iface-persist",      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->persist)),
                 NULL);
 }
@@ -374,6 +390,7 @@ connection_dialog_init (void)
   /* wireless */
   gcd->essid   = gst_dialog_get_widget (tool->main_dialog, "connection_essid");
   gcd->wep_key = gst_dialog_get_widget (tool->main_dialog, "connection_wep_key");
+  gcd->key_type_combo = gst_dialog_get_widget (tool->main_dialog, "connection_wep_key_type");
 
   /* plip */
   gcd->local_address  = gst_dialog_get_widget (tool->main_dialog, "connection_local_address");
@@ -389,6 +406,7 @@ connection_dialog_init (void)
   gcd->volume       = gst_dialog_get_widget (tool->main_dialog, "connection_volume");
   gcd->dial_type    = gst_dialog_get_widget (tool->main_dialog, "connection_dial_type");
   gcd->default_gw   = gst_dialog_get_widget (tool->main_dialog, "connection_default_gw");
+  gcd->peerdns      = gst_dialog_get_widget (tool->main_dialog, "connection_peerdns");
   gcd->persist      = gst_dialog_get_widget (tool->main_dialog, "connection_persist");
 
   gcd->wireless_frame = gst_dialog_get_widget (tool->main_dialog, "connection_wireless");
@@ -396,7 +414,6 @@ connection_dialog_init (void)
   gcd->plip_frame     = gst_dialog_get_widget (tool->main_dialog, "connection_plip");
   gcd->modem_frame    = gst_dialog_get_widget (tool->main_dialog, "connection_modem");
 
-  connection_bootproto_init (gcd);
   connection_essids_combo_init (GTK_COMBO_BOX_ENTRY (gcd->essid));
 
   return gcd;
