@@ -30,6 +30,7 @@
 #include "gst.h"
 
 #include "table.h"
+#include "service.h"
 #include "callbacks.h"
 
 GtkActionEntry popup_menu_items[] = {
@@ -43,16 +44,25 @@ const gchar *ui_description =
 	"  </popup>"
 	"</ui>";
 
-typedef struct TreeItem_ TreeItem;
+typedef struct _TreeItem TreeItem;
 	
-struct TreeItem_ {
-	gchar *service;
-	gboolean active;
-	gint priority;
+struct _TreeItem {
+	gboolean   active;
+	gchar     *desc;
+	GdkPixbuf *image;
+	gboolean   dangerous;
 };
 
 extern GstTool *tool;
 
+static void
+free_tree_item (TreeItem *item)
+{
+	g_free (item->desc);
+	g_object_unref (item->image);
+	g_free (item);
+}
+
 static GtkWidget*
 table_popup_menu_create (GtkTreeView *treeview)
 {
@@ -76,48 +86,54 @@ table_popup_menu_create (GtkTreeView *treeview)
 static void
 add_columns (GtkTreeView *treeview)
 {
-	gint i, *col;
-	gchar *label;
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
 
+	column = gtk_tree_view_column_new ();
+
+	/* Checkbox */
 	renderer = gtk_cell_renderer_toggle_new ();
-	gtk_tree_view_insert_column_with_attributes (treeview, -1,
-						     _("Active"),
-						     renderer,
-						     "active", COL_ACTIVE,
-						     NULL);
 
 	g_signal_connect (G_OBJECT (renderer), "toggled",
 			  G_CALLBACK (on_service_toggled), tool);
-	
-	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes (_("Service"),
-							   renderer,
-							   "text", COL_SERVICE,
-							   NULL);
-	gtk_tree_view_column_set_sort_column_id (column, COL_SERVICE);
-	gtk_tree_view_insert_column (treeview, column, -1);
-	
-	column = gtk_tree_view_column_new_with_attributes ("priority",
-							   renderer,
-							   "text", COL_PRIORITY,
-							   NULL);
-	gtk_tree_view_column_set_sort_column_id (column, COL_PRIORITY);
-	gtk_tree_view_column_set_sort_indicator (column, FALSE);
-	gtk_tree_view_column_set_visible (column, FALSE);
 
+	g_object_set (G_OBJECT (renderer), "xpad", 12, NULL);
+
+	gtk_tree_view_insert_column_with_attributes (treeview,
+						     -1, "",
+						     renderer,
+						     "active", COL_ACTIVE,
+						     NULL);
+	/* Image */
+	renderer = gtk_cell_renderer_pixbuf_new ();
+	gtk_tree_view_column_pack_start (column, renderer, FALSE);
+	gtk_tree_view_column_set_attributes (column, renderer,
+					     "pixbuf", COL_IMAGE,
+					     NULL);
+
+	/* Text */
+	renderer = gtk_cell_renderer_text_new ();
+	gtk_tree_view_column_pack_start (column, renderer, TRUE);
+	gtk_tree_view_column_set_attributes (column, renderer,
+					     "markup", COL_DESC,
+					     NULL);
+
+	g_object_set (G_OBJECT (renderer), "yalign", 0, NULL);
 	gtk_tree_view_insert_column (treeview, column, -1);
+
+	gtk_tree_view_column_set_sort_column_id (column, 1);
+	gtk_tree_view_column_clicked (column);
 }
 
 static GtkTreeModel*
 create_model (void)
 {
-	GtkTreeStore *model;
+	GtkListStore *model;
 
-	model = gtk_tree_store_new (COL_LAST,
+	model = gtk_list_store_new (COL_LAST,
 				    G_TYPE_BOOLEAN,
 				    G_TYPE_STRING,
+				    GDK_TYPE_PIXBUF,
 				    G_TYPE_INT,
 				    G_TYPE_POINTER);
 	return GTK_TREE_MODEL(model);
@@ -126,7 +142,7 @@ create_model (void)
 void
 table_create (void)
 {
-	GtkWidget        *runlevel_table = gst_dialog_get_widget (tool->main_dialog, "runlevel_table");
+	GtkWidget        *runlevel_table = gst_dialog_get_widget (tool->main_dialog, "services_list");
 	GtkTreeModel     *model;
 	GtkTreeSelection *selection;
 	GtkWidget        *menu;
@@ -138,92 +154,76 @@ table_create (void)
 	
 	add_columns (GTK_TREE_VIEW (runlevel_table));
 
+	/* FIXME: remove this until I figure out what to do with the advanced properties
+
 	menu = table_popup_menu_create (GTK_TREE_VIEW (runlevel_table));
 	g_signal_connect (G_OBJECT (runlevel_table), "button_press_event",
 			  G_CALLBACK (on_table_button_press_event), menu);
 	g_signal_connect (G_OBJECT (runlevel_table), "popup_menu",
 			  G_CALLBACK (on_table_popup_menu), menu);
+	*/
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (runlevel_table));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
-	g_signal_connect (G_OBJECT (selection), "changed",
-			  G_CALLBACK (on_services_table_select_row), NULL);
-}
-
-static gchar*
-table_value_service(xmlNodePtr node)
-{
-	gchar *buf, *name, *script;
-	g_return_val_if_fail (node != NULL, NULL);
-
-	name = gst_xml_get_child_content (node, "name");
-	script = gst_xml_get_child_content (node, "script");
-	
-	if (name == NULL)
-		buf = script;
-	else {
-		buf = g_strdup_printf ("%s - %s", script, name);
-		g_free (name);
-		g_free (script);
-	}
-	
-	return buf;
 }
 
 static gboolean
 table_value_active (xmlNodePtr node, gchar *runlevel)
 {
-	gboolean value = FALSE;
 	xmlNodePtr runlevels = gst_xml_element_find_first (node, "runlevels");
 	xmlNodePtr rl;
 	gchar *str, *action;
+	gboolean value, found;
 
-	if (runlevels) {
-		for (rl = gst_xml_element_find_first (runlevels, "runlevel");
-		     rl != NULL;
-		     rl = gst_xml_element_find_next (rl, "runlevel"))
-		{
-			str = gst_xml_get_child_content (rl, "number");
+	if (!runlevels)
+		return FALSE;
 
-			if (str && runlevel && (strcmp (str, runlevel) == 0)) {
-				action = gst_xml_get_child_content (rl, "action");
+	value = found = FALSE;
+	rl = gst_xml_element_find_first (runlevels, "runlevel");
 
-				if (strcmp (action, "start") == 0)
-					value = TRUE;
+	while (!found && rl) {
+		str = gst_xml_get_child_content (rl, "name");
 
-				g_free (action);
-			}
+		if (str && runlevel && (strcmp (str, runlevel) == 0)) {
+			action = gst_xml_get_child_content (rl, "action");
+			found  = TRUE;
 
-			g_free (str);
+			if (action && strcmp (action, "start") == 0)
+				value = TRUE;
+			else
+				value = FALSE;
+
+			g_free (action);
 		}
+
+		g_free (str);
+		rl = gst_xml_element_find_next (rl, "runlevel");
 	}
 
 	return value;
 }
 
-static gint
-table_value_priority (xmlNodePtr node)
-{
-	gchar *str = gst_xml_get_child_content (node, "priority");
-	gint   val;
-
-	if (!str)
-		return 0;
-
-	val = (gint) g_strtod (str, NULL);
-	g_free (str);
-	
-	return val;
-}
-
 static TreeItem*
 get_node_data (xmlNodePtr service, gchar *runlevel)
 {
-	TreeItem *item = g_malloc (sizeof(TreeItem));
+	const ServiceDescription *desc;
+	TreeItem *item;
+	gchar *script, *role;
 
-	item->service = table_value_service (service);
-	item->active = table_value_active (service, runlevel);
-	item->priority = table_value_priority (service);
+	script = gst_xml_get_child_content (service, "script");
+	role   = gst_xml_get_child_content (service, "role");
+	desc   = service_search (role);
+
+	if (!desc)
+		return NULL;
+
+	item = g_malloc0 (sizeof (TreeItem));
+	item->desc      = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s (<i>%s</i>)</span>\n%s",
+					   _(desc->description), script, _(desc->long_description));
+
+	item->image     = gtk_icon_theme_load_icon (tool->icon_theme, desc->icon, 48, 0, NULL);
+	item->active    = table_value_active (service, runlevel);
+	item->dangerous = desc->dangerous;
 
 	return item;
 }
@@ -231,41 +231,37 @@ get_node_data (xmlNodePtr service, gchar *runlevel)
 void 
 table_populate (xmlNodePtr root, gchar *runlevel)
 {
-	xmlNodePtr service,services;
-	GtkTreeIter iter;
-	GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(gst_dialog_get_widget (tool->main_dialog, "runlevel_table")));
-	
+	xmlNodePtr    service,services;
+	GtkTreeView  *treeview;
+	GtkTreeModel *model;
+	GtkTreeIter   iter;
+
 	g_return_if_fail (root != NULL);
 	
+	treeview = GTK_TREE_VIEW (gst_dialog_get_widget (tool->main_dialog, "services_list"));
+	model    = gtk_tree_view_get_model (treeview);
 	services = gst_xml_element_find_first (root, "services");
 	
-	for (service = gst_xml_element_find_first (services, "service"); service != NULL; service = gst_xml_element_find_next (service, "service"))
-	{
+	for (service = gst_xml_element_find_first (services, "service");
+	     service; service = gst_xml_element_find_next (service, "service")) {
 		TreeItem *item;
 
 		item = get_node_data (service, runlevel);
 
-		gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
-		gtk_tree_store_set (GTK_TREE_STORE (model),
+		/* FIXME: should do something with unknown services? */
+		if (!item)
+			continue;
+
+		gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+		gtk_list_store_set (GTK_LIST_STORE (model),
 		                    &iter,
-		                    COL_ACTIVE, item->active,
-		                    COL_SERVICE, item->service,
-				    COL_PRIORITY, item->priority,
-				    COL_POINTER, service,
+		                    COL_ACTIVE,    item->active,
+		                    COL_DESC,      item->desc,
+				    COL_IMAGE,     item->image,
+				    COL_POINTER,   service,
+				    COL_DANGEROUS, item->dangerous,
 		                    -1);
 
-		g_free (item->service);
-		g_free (item);
+		free_tree_item (item);
 	}
-	
-}
-
-void
-table_clear (void)
-{
-	GtkWidget *runlevel_table = gst_dialog_get_widget (tool->main_dialog, "runlevel_table");
-	GtkTreeModel *model;
-
-	model = gtk_tree_view_get_model (GTK_TREE_VIEW (runlevel_table));
-	gtk_tree_store_clear (GTK_TREE_STORE (model));
 }
