@@ -41,15 +41,27 @@ extern GstTool *tool;
 GtkWidget *users_table;
 GArray *users_array;
 
-TableConfig users_table_config [] = {
-	/*Column name,          Adv_state_show,	Basic_state_show*/
-	{ N_("User"),		TRUE,		TRUE},
-	{ N_("UID"),		TRUE,		FALSE},
-	{ N_("Home"),		TRUE,		FALSE},
-	{ N_("Shell"),		TRUE,		FALSE},
-	{ N_("Comments"),	FALSE,		TRUE},
-	{ N_("Group"),		FALSE,		TRUE},
-	{NULL}
+typedef struct _UserTreeItem UserTreeItem;
+
+struct _UserTreeItem
+{
+	gchar *login;
+	guint UID;
+	gchar *home;
+	gchar *shell;
+	gchar *comment;
+	
+	UserTreeItem *children;
+};
+
+
+gchar *users_table_columns [] = {
+	N_("User"),
+	N_("Home"),
+	N_("User details"),
+	N_("UID"),
+	N_("Shell"),
+	NULL
 };
 
 static void
@@ -57,24 +69,20 @@ add_user_columns (GtkTreeView *treeview)
 {
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *renderer;
-	TableConfig *i;
-	guint j;
+	guint i;
 	
-	for (i = users_table_config, j = 0; 
-	     i->name != NULL; 
-	     i++, j++)
-	{	
+	for (i = 0; users_table_columns [i] != NULL; i++) {
+
 		renderer = gtk_cell_renderer_text_new ();
-	
-		column = gtk_tree_view_column_new_with_attributes (_(i->name),
+
+		column = gtk_tree_view_column_new_with_attributes (_(users_table_columns [i]),
 	                                                           renderer,
-	                                                           "text",
-	                                                           j,
+	                                                           "text", i,
 	                                                           NULL);
 		gtk_tree_view_column_set_resizable (column, TRUE);
-		gtk_tree_view_column_set_sort_column_id (column, j);
+		gtk_tree_view_column_set_sort_column_id (column, i);
 
-		gtk_tree_view_insert_column (GTK_TREE_VIEW (users_table), column, j);
+		gtk_tree_view_insert_column (GTK_TREE_VIEW (users_table), column, i);
 	}
 }
 
@@ -85,24 +93,21 @@ create_users_model (void)
 	
 	model = gtk_tree_store_new (COL_USER_LAST,
 	                            G_TYPE_STRING,
+	                            G_TYPE_STRING,
+	                            G_TYPE_STRING,
+				    G_TYPE_INT,
+				    G_TYPE_STRING,
 	                            G_TYPE_INT,
-	                            G_TYPE_STRING,
-	                            G_TYPE_STRING,
-	                            G_TYPE_STRING,
-	                            G_TYPE_STRING,
-	                            G_TYPE_INT,
-	                            G_TYPE_POINTER);
+				    G_TYPE_POINTER);
 	return GTK_TREE_MODEL (model);
 }
 
-
-
 void
 create_users_table (void)
 {
 	GtkTreeSelection *selection;
-	GtkTreeModel *model;
-	GtkItemFactory *item_factory;
+	GtkTreeModel     *model;
+	GtkWidget        *popup;
 	
 	model = create_users_model ();
 	
@@ -116,14 +121,15 @@ create_users_table (void)
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (users_table));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
 
-	item_factory = popup_item_factory_create (users_table);
+	popup = popup_menu_create (users_table);
 
 	g_signal_connect (G_OBJECT (selection), "changed",
 			  G_CALLBACK (on_table_clicked),
 			  (gpointer) users_table);
 	g_signal_connect (G_OBJECT (users_table), "button_press_event",
-			  G_CALLBACK (on_table_button_press),
-			  (gpointer) item_factory);
+			  G_CALLBACK (on_table_button_press), popup);
+	g_signal_connect (G_OBJECT (users_table), "popup-menu",
+			  G_CALLBACK (on_table_popup_menu), popup);
 }
 
 static char*
@@ -136,6 +142,7 @@ users_table_value (xmlNodePtr node, gchar *key)
 static UserTreeItem*
 get_user_node_data (xmlNodePtr user)
 {
+	gint i, j, len;
         UserTreeItem *item = g_malloc (sizeof(UserTreeItem));
 	
 	item->login = users_table_value (user, "login");
@@ -143,11 +150,37 @@ get_user_node_data (xmlNodePtr user)
 	item->home = users_table_value (user, "home");
 	item->shell = users_table_value (user, "shell");
 	item->comment = users_table_value (user, "comment");
-	item->group = user_value_group_peek (user);
-        
+
+	/* Remove ending and double commas from comment */
+	len = strlen (item->comment);
+	j = 0;
+	for (i = 0; i < len; i++) {
+		if (item->comment[i] == ',') 	{
+			if (item->comment[i+1] == '\0') {
+				item->comment[j] = '\0';
+			} else 	if (item->comment[i+1] != ',') {
+				item->comment[j] = item->comment[i];
+				j++;						
+			}
+		} else {
+			item->comment[j] = item->comment[i];
+			j++;
+		}	
+	}
+	item->comment[j] = '\0';
         return item;
 }
 
+static void 
+free_user_tree_item (UserTreeItem *user_tree_item)
+{
+	g_free(user_tree_item->login);
+	g_free(user_tree_item->home);
+	g_free(user_tree_item->shell);
+	g_free(user_tree_item->comment);
+
+	g_free(user_tree_item);
+}
 void
 populate_users_table (void)
 {
@@ -155,75 +188,34 @@ populate_users_table (void)
 	xmlNodePtr root = get_root_node (NODE_USER);
 	GtkTreeIter iter;
 	xmlNodePtr user;
-	
+	UserTreeItem *item;
+
 	g_return_if_fail (model != NULL);
 	g_return_if_fail (root != NULL);
+
+	gtk_tree_store_clear (GTK_TREE_STORE (model));
 	
 	users_array = g_array_new (FALSE, FALSE, sizeof (xmlNodePtr));
-	
-	for (user = gst_xml_element_find_first (root, "user"); user != NULL; user = gst_xml_element_find_next (user, "user"))
-	{
-		if (check_node_visibility (user))
-		{
-			UserTreeItem *item;
+
+	for (user = gst_xml_element_find_first (root, "user"); user != NULL; user = gst_xml_element_find_next (user, "user")) {
+		if (check_node_visibility (user)) {
 			item = get_user_node_data (user);
-		
+
 			gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
 			gtk_tree_store_set (GTK_TREE_STORE (model),
 			                    &iter,
 					    COL_USER_LOGIN, item->login,
-					    COL_USER_UID, item->UID,
 					    COL_USER_HOME, item->home,
-					    COL_USER_SHELL, item->shell,
 					    COL_USER_COMMENT, item->comment,
-					    COL_USER_GROUP, item->group,
+					    COL_USER_UID, item->UID,
+					    COL_USER_SHELL, item->shell,
 			                    COL_USER_POINTER, user,
 					    -1);
+			free_user_tree_item (item);
 		}
 	}
 }
 
-void
-update_users_table_complexity (GstDialogComplexity complexity)
-{
-	GtkTreeView *u_table = GTK_TREE_VIEW (users_table);
-	GtkTreeView *g_table = GTK_TREE_VIEW (gst_dialog_get_widget (tool->main_dialog, "groups_table"));
-	GtkTreeViewColumn *column;
-	TableConfig *i;
-	guint j;
-
-	gtk_tree_selection_unselect_all (gtk_tree_view_get_selection (u_table));
-	gtk_tree_selection_unselect_all (gtk_tree_view_get_selection (g_table));
-
-	actions_set_sensitive (NODE_USER, FALSE);
-	actions_set_sensitive (NODE_GROUP, FALSE);
-	
-	switch (complexity) {
-	case GST_DIALOG_BASIC:
-		for (i = users_table_config, j=0;
-		     i->name != NULL;
-		     i++, j++)
-		{
-			column = gtk_tree_view_get_column (u_table, j);
-			gtk_tree_view_column_set_visible (column, i->basic_state_showable);
-		}
-		break;
-	case GST_DIALOG_ADVANCED:
-		for (i = users_table_config, j=0;
-		     i->name != NULL;
-		     i++, j++)
-		{
-			column = gtk_tree_view_get_column (u_table, j);
-			gtk_tree_view_column_set_visible (column, i->advanced_state_showable);
-		}
-		break;
-	default:
-		g_warning ("tables_update_complexity: Unsupported complexity.");
-		return;
-	}
-
-}
-
 static void
 clear_users_table (void)
 {
@@ -237,21 +229,28 @@ users_table_set_cursor (gchar *old_login)
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	gchar *login;
+	gboolean valid;
 	
 	if (old_login == NULL)
 		return;	
+
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (users_table));
-	if (gtk_tree_model_get_iter_first (model, &iter) != TRUE)
-		return;
-	do {
+	valid = gtk_tree_model_get_iter_first (model, &iter);
+
+	while (valid) {
 		gtk_tree_model_get (model, &iter, 0, &login, -1);
-		if (strcmp (old_login, login) == 0 ) 
-		{
+
+		if (strcmp (old_login, login) == 0 ) {
 			GtkTreePath *path = gtk_tree_model_get_path (model, &iter);
 			gtk_tree_view_set_cursor (GTK_TREE_VIEW (users_table), path, NULL, FALSE);
 			gtk_tree_path_free (path);
+			g_free (login);
+
+			return;
 		}
-	} while (gtk_tree_model_iter_next (model, &iter) != FALSE);
+		g_free (login);
+		valid = gtk_tree_model_iter_next (model, &iter);
+	}
 }
 
 void
